@@ -49,6 +49,8 @@
 #define gmtime_r(tp,tmp) (gmtime(tp)?(*(tmp)=*gmtime(tp),(tmp)):0)
 #endif
 
+G_DEFINE_TYPE (ECalBackendMAPI, e_cal_backend_mapi, E_TYPE_CAL_BACKEND_SYNC)
+
 typedef struct {
 	GCond *cond;
 	GMutex *mutex;
@@ -87,7 +89,81 @@ struct _ECalBackendMAPIPrivate {
 	guint 			timeout_id;
 	GThread 		*dthread;
 	SyncDelta 		*dlock;
-	GSList 			*cache_keys;
+};
+
+/* this is a list of all known calendar MAPI tag IDs;
+   if you add new add it here too, otherwise it may not be fetched */
+static uint32_t known_cal_mapi_ids[] = {
+	PR_7BIT_DISPLAY_NAME_UNICODE,
+	PR_ADDRTYPE_UNICODE,
+	PR_ATTACH_DATA_BIN,
+	PR_ATTACH_FILENAME_UNICODE,
+	PR_ATTACH_LONG_FILENAME_UNICODE,
+	PR_ATTACH_METHOD,
+	PR_BODY,
+	PR_BODY_UNICODE,
+	PR_CONVERSATION_TOPIC_UNICODE,
+	PR_CREATION_TIME,
+	PR_DISPLAY_NAME_UNICODE,
+	PR_DISPLAY_TYPE,
+	PR_END_DATE,
+	PR_FID,
+	PR_GIVEN_NAME_UNICODE,
+	PR_HTML,
+	PR_ICON_INDEX,
+	PR_IMPORTANCE,
+	PR_LAST_MODIFICATION_TIME,
+	PR_MESSAGE_CLASS,
+	PR_MESSAGE_FLAGS,
+	PR_MID,
+	PR_MSG_EDITOR_FORMAT,
+	PR_NORMALIZED_SUBJECT_UNICODE,
+	PR_OBJECT_TYPE,
+	PR_OWNER_APPT_ID,
+	PR_PRIORITY,
+	PR_PROCESSED,
+	PR_RCVD_REPRESENTING_ADDRTYPE_UNICODE,
+	PR_RCVD_REPRESENTING_EMAIL_ADDRESS_UNICODE,
+	PR_RCVD_REPRESENTING_NAME_UNICODE,
+	PR_RECIPIENT_DISPLAY_NAME_UNICODE,
+	PR_RECIPIENTS_FLAGS,
+	PR_RECIPIENT_TRACKSTATUS,
+	PR_RECIPIENT_TYPE,
+	PR_RENDERING_POSITION,
+	PR_RESPONSE_REQUESTED,
+	PR_RTF_IN_SYNC,
+	PR_SENDER_ADDRTYPE_UNICODE,
+	PR_SENDER_EMAIL_ADDRESS_UNICODE,
+	PR_SENDER_NAME_UNICODE,
+	PR_SEND_INTERNET_ENCODING,
+	PR_SENSITIVITY,
+	PR_SENT_REPRESENTING_ADDRTYPE_UNICODE,
+	PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE,
+	PR_SENT_REPRESENTING_NAME_UNICODE,
+	PR_SMTP_ADDRESS_UNICODE,
+	PR_START_DATE,
+	PR_SUBJECT_UNICODE,
+	PROP_TAG(PT_BINARY, 0x0003),
+	PROP_TAG(PT_BINARY, 0x0023),
+	PROP_TAG(PT_BINARY, 0x8216),
+	PROP_TAG(PT_BINARY, 0x825E),
+	PROP_TAG(PT_BINARY, 0x825F),
+	PROP_TAG(PT_BOOLEAN, 0x8126),
+	PROP_TAG(PT_BOOLEAN, 0x8215),
+	PROP_TAG(PT_BOOLEAN, 0x8223),
+	PROP_TAG(PT_BOOLEAN, 0x8503),
+	PROP_TAG(PT_DOUBLE, 0x8102),
+	PROP_TAG(PT_LONG, 0x8101),
+	PROP_TAG(PT_LONG, 0x8201),
+	PROP_TAG(PT_LONG, 0x8205),
+	PROP_TAG(PT_STRING8, 0x8208),
+	PROP_TAG(PT_SYSTIME, 0x8104),
+	PROP_TAG(PT_SYSTIME, 0x8105),
+	PROP_TAG(PT_SYSTIME, 0x810F),
+	PROP_TAG(PT_SYSTIME, 0x820D),
+	PROP_TAG(PT_SYSTIME, 0x820E),
+	PROP_TAG(PT_SYSTIME, 0x8502),
+	PROP_TAG(PT_SYSTIME, 0x8560)
 };
 
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_SYNC
@@ -381,7 +457,7 @@ get_element_type (icalcomponent_kind kind)
 static void 
 notify_progress (ECalBackendMAPI *cbmapi, guint64 index, guint64 total)
 {
-	guint percent = ((float)index/total) * 100 ;
+	guint percent = ((float) index/total) * 100 ;
 	gchar *progress_string;
 
 	if (percent > 100)
@@ -389,7 +465,7 @@ notify_progress (ECalBackendMAPI *cbmapi, guint64 index, guint64 total)
 
 	/* To translators: This message is displayed on the status bar when calendar/tasks/memo items are being fetched from the server. */
 	progress_string = g_strdup_printf (_("Loading items in folder %s"), 
-				e_source_get_property (e_cal_backend_get_source (E_CAL_BACKEND (cbmapi)), "name")); 
+				e_source_peek_name (e_cal_backend_get_source (E_CAL_BACKEND (cbmapi))));
 
 	e_cal_backend_notify_view_progress (E_CAL_BACKEND (cbmapi), progress_string, percent);
 
@@ -490,19 +566,60 @@ mapi_cal_get_changes_cb (FetchItemsCallbackData *item_data, gpointer data)
 	return TRUE;
 }
 
+struct deleted_items_data {
+	ECalBackendMAPI *cbmapi;
+	GSList *cache_keys;
+	GSList *unknown_mids; /* MIDs of items not in the cache */
+};
+
 static gboolean
 handle_deleted_items_cb (FetchItemsCallbackData *item_data, gpointer data)
 {
-	const mapi_id_t mid = item_data->mid; 
-	ECalBackendMAPI *cbmapi	= data;
-	ECalBackendMAPIPrivate *priv = cbmapi->priv;
+	const mapi_id_t mid = item_data->mid;
+	struct deleted_items_data *did = data;
 	gchar *tmp = NULL;
 	GSList *cache_comp_uid = NULL;
+	gboolean need_refetch = FALSE;
+
+	g_return_val_if_fail (did != NULL, FALSE);
 
 	tmp = exchange_mapi_util_mapi_id_to_string (mid);
-	cache_comp_uid = g_slist_find_custom (priv->cache_keys, tmp, (GCompareFunc) (g_ascii_strcasecmp));
-	if (cache_comp_uid != NULL)
-		priv->cache_keys = g_slist_remove_link (priv->cache_keys, cache_comp_uid);
+	cache_comp_uid = g_slist_find_custom (did->cache_keys, tmp, (GCompareFunc) (g_ascii_strcasecmp));
+	if (cache_comp_uid != NULL) {
+		ECalBackendMAPIPrivate *priv = did->cbmapi->priv;
+		ECalComponent *comp;
+
+		comp = e_cal_backend_cache_get_component (priv->cache, cache_comp_uid->data, NULL);
+		if (comp) {
+			struct icaltimetype *last_mod = NULL;
+			struct timeval t;
+
+			e_cal_component_get_last_modified (comp, &last_mod);
+			if (!last_mod) {
+				need_refetch = TRUE;
+			} else if (get_mapi_SPropValue_array_date_timeval (&t, item_data->properties, PR_LAST_MODIFICATION_TIME) == MAPI_E_SUCCESS
+			    && icaltime_compare (icaltime_from_timet_with_zone (t.tv_sec, 0, icaltimezone_get_utc_timezone ()), *last_mod) != 0) {
+				need_refetch = TRUE;
+			}
+
+			if (last_mod)
+				e_cal_component_free_icaltimetype (last_mod);
+
+			g_object_unref (comp);
+		}
+
+		did->cache_keys = g_slist_remove_link (did->cache_keys, cache_comp_uid);
+	} else {
+		/* fetch it, as it is not in the cache */
+		need_refetch = TRUE;
+	}
+
+	if (need_refetch) {
+		mapi_id_t *nmid = g_new (mapi_id_t, 1);
+
+		*nmid = mid;
+		did->unknown_mids = g_slist_prepend (did->unknown_mids, nmid);
+	}
 
 	g_free (tmp);
 	return TRUE;
@@ -531,6 +648,7 @@ get_deltas (gpointer handle)
 	struct mapi_SRestriction res;
 	gboolean use_restriction = FALSE;
 	GSList *ls = NULL;
+	struct deleted_items_data did;
 
 	if (!handle)
 		return FALSE;
@@ -547,6 +665,7 @@ get_deltas (gpointer handle)
 	serv_time = e_cal_backend_cache_get_server_utc_time (priv->cache);
 	if (serv_time)
 		itt_cache = icaltime_from_string (serv_time); 
+
 	if (!icaltime_is_null_time (itt_cache)) {
 		struct SPropValue sprop;
 		struct timeval t;
@@ -560,8 +679,7 @@ get_deltas (gpointer handle)
 		t.tv_usec = 0;
 		set_SPropValue_proptag_date_timeval (&sprop, PR_LAST_MODIFICATION_TIME, &t);
 		cast_mapi_SPropValue (&(res.res.resProperty.lpProp), &sprop);
-	} else
-		g_warning ("Cache time-stamp not found."); 
+	}
 
 	itt_current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 	current_time = icaltime_as_timet_with_zone (itt_current, icaltimezone_get_utc_timezone ());
@@ -574,7 +692,7 @@ get_deltas (gpointer handle)
 	/* FIXME: GetProps does not seem to work for tasks :-( */
 	if (kind == ICAL_VTODO_COMPONENT) {
 		if (!exchange_mapi_connection_fetch_items (priv->fid, use_restriction ? &res : NULL, NULL,
-						NULL, 0, NULL, NULL, 
+						known_cal_mapi_ids, G_N_ELEMENTS (known_cal_mapi_ids), NULL, NULL, 
 						mapi_cal_get_changes_cb, cbmapi, 
 						MAPI_OPTIONS_FETCH_ALL)) {
 			/* FIXME: String : We need to restart evolution-data-server */
@@ -608,21 +726,23 @@ get_deltas (gpointer handle)
 	/* e_cal_backend_cache_get_keys returns a list of all the keys. 
 	 * The items in the list are pointers to internal data, 
 	 * so should not be freed, only the list should. */
-	priv->cache_keys = e_cal_backend_cache_get_keys (priv->cache);
+	did.cbmapi = cbmapi;
+	did.cache_keys = e_cal_backend_cache_get_keys (priv->cache);
+	did.unknown_mids = NULL;
 	if (!exchange_mapi_connection_fetch_items (priv->fid, NULL, NULL,
 						cal_IDList, G_N_ELEMENTS (cal_IDList), 
 						NULL, NULL, 
-						handle_deleted_items_cb, cbmapi, 
+						handle_deleted_items_cb, &did, 
 						0)) {
 		/* FIXME: String : We need to restart evolution-data-server */
 		e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Error fetching changes from the server."));
-		priv->cache_keys = NULL;
+		g_slist_free (did.cache_keys);
 		g_static_mutex_unlock (&updating);
 		return FALSE;
 	}
 
 	e_file_cache_freeze_changes (E_FILE_CACHE (priv->cache));
-	for (ls = priv->cache_keys; ls ; ls = g_slist_next (ls)) {
+	for (ls = did.cache_keys; ls ; ls = g_slist_next (ls)) {
 		ECalComponent *comp = NULL;
 		icalcomponent *icalcomp = NULL;
 
@@ -648,11 +768,56 @@ get_deltas (gpointer handle)
 	}
 	e_file_cache_thaw_changes (E_FILE_CACHE (priv->cache));
 
-//	g_slist_free (priv->cache_keys); 
-	priv->cache_keys = NULL;
+	g_slist_free (did.cache_keys);
+
+	if (did.unknown_mids) {
+		gint i;
+		struct mapi_SRestriction_or *or_res = g_new0 (struct mapi_SRestriction_or, g_slist_length (did.unknown_mids));
+
+		for (i = 0, ls = did.unknown_mids; ls; i++, ls = ls->next) {
+			mapi_id_t *pmid = ls->data;
+
+			or_res[i].rt = RES_PROPERTY;
+			or_res[i].res.resProperty.relop = RELOP_EQ;
+			or_res[i].res.resProperty.ulPropTag = PR_MID;
+			or_res[i].res.resProperty.lpProp.ulPropTag = PR_MID;
+			or_res[i].res.resProperty.lpProp.value.dbl = *pmid;
+		}
+
+		memset (&res, 0, sizeof (struct mapi_SRestriction));
+		res.rt = RES_OR;
+		res.res.resOr.cRes = g_slist_length (did.unknown_mids);
+		res.res.resOr.res = or_res;
+
+		g_slist_foreach (did.unknown_mids, (GFunc) g_free, NULL);
+		g_slist_free (did.unknown_mids);
+
+		if (kind == ICAL_VTODO_COMPONENT) {
+			if (!exchange_mapi_connection_fetch_items (priv->fid, &res, NULL,
+						known_cal_mapi_ids, G_N_ELEMENTS (known_cal_mapi_ids), NULL, NULL,
+						mapi_cal_get_changes_cb, cbmapi,
+						MAPI_OPTIONS_FETCH_ALL)) {
+				e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Error fetching changes from the server."));
+				g_static_mutex_unlock (&updating);
+				g_free (or_res);
+				return FALSE;
+			}
+		} else if (!exchange_mapi_connection_fetch_items (priv->fid, &res, NULL,
+						cal_GetPropsList, G_N_ELEMENTS (cal_GetPropsList),
+						exchange_mapi_cal_util_build_name_id, GINT_TO_POINTER(kind),
+						mapi_cal_get_changes_cb, cbmapi,
+						MAPI_OPTIONS_FETCH_ALL)) {
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Error fetching changes from the server."));
+			g_free (or_res);
+			g_static_mutex_unlock (&updating);
+			return FALSE;
+		}
+
+		g_free (or_res);
+	}
 
 	g_static_mutex_unlock (&updating);
-	return TRUE;        
+	return TRUE;
 }
 
 static ECalBackendSyncStatus
@@ -967,7 +1132,7 @@ populate_cache (ECalBackendMAPI *cbmapi)
 	/* FIXME: GetProps does not seem to work for tasks :-( */
 	if (kind == ICAL_VTODO_COMPONENT) {
 		if (!exchange_mapi_connection_fetch_items (priv->fid, NULL, NULL,
-						NULL, 0, NULL, NULL, 
+						known_cal_mapi_ids, G_N_ELEMENTS (known_cal_mapi_ids), NULL, NULL, 
 						mapi_cal_cache_create_cb, cbmapi, 
 						MAPI_OPTIONS_FETCH_ALL)) {
 			e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Could not create cache file"));
@@ -1222,6 +1387,19 @@ e_cal_backend_mapi_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_
 		return status;
 }
 
+static uint32_t req_props_list[] = {
+	PR_OWNER_APPT_ID,
+	PROP_TAG(PT_LONG, 0x8201),
+	PROP_TAG(PT_BINARY, 0x0023),
+	PROP_TAG(PT_BINARY, 0x0003),
+	PR_SENT_REPRESENTING_NAME_UNICODE,
+	PR_SENT_REPRESENTING_ADDRTYPE_UNICODE,
+	PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE,
+	PR_SENDER_NAME_UNICODE,
+	PR_SENDER_ADDRTYPE_UNICODE,
+	PR_SENDER_EMAIL_ADDRESS_UNICODE
+};	
+
 static gboolean 
 capture_req_props (FetchItemsCallbackData *item_data, gpointer data)
 {
@@ -1237,12 +1415,12 @@ capture_req_props (FetchItemsCallbackData *item_data, gpointer data)
 		cbdata->appt_seq = *ui32;
 	cbdata->cleanglobalid = (const struct Binary_r *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_BINARY, 0x0023));
 	cbdata->globalid = (const struct Binary_r *)find_mapi_SPropValue_data(properties, PROP_TAG(PT_BINARY, 0x0003));
-	cbdata->username = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_NAME);
-	cbdata->useridtype = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_ADDRTYPE);
-	cbdata->userid = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_EMAIL_ADDRESS);
-	cbdata->ownername = exchange_mapi_util_find_array_propval (properties, PR_SENDER_NAME);
-	cbdata->owneridtype = exchange_mapi_util_find_array_propval (properties, PR_SENDER_ADDRTYPE);
-	cbdata->ownerid = exchange_mapi_util_find_array_propval (properties, PR_SENDER_EMAIL_ADDRESS);
+	cbdata->username = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_NAME_UNICODE);
+	cbdata->useridtype = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_ADDRTYPE_UNICODE);
+	cbdata->userid = exchange_mapi_util_find_array_propval (properties, PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE);
+	cbdata->ownername = exchange_mapi_util_find_array_propval (properties, PR_SENDER_NAME_UNICODE);
+	cbdata->owneridtype = exchange_mapi_util_find_array_propval (properties, PR_SENDER_ADDRTYPE_UNICODE);
+	cbdata->ownerid = exchange_mapi_util_find_array_propval (properties, PR_SENDER_EMAIL_ADDRESS_UNICODE);
 
 	return TRUE;
 }
@@ -1262,7 +1440,7 @@ get_server_data (ECalBackendMAPI *cbmapi, icalcomponent *comp, struct cbdata *cb
 	uid = icalcomponent_get_uid (comp);
 	exchange_mapi_util_mapi_id_from_string (uid, &mid);
 	if (exchange_mapi_connection_fetch_item (priv->fid, mid, 
-					NULL, 0, 
+					req_props_list, G_N_ELEMENTS (req_props_list),
 					NULL, NULL, 
 					capture_req_props, cbdata, 
 					MAPI_OPTIONS_FETCH_GENERIC_STREAMS))
@@ -1282,7 +1460,7 @@ get_server_data (ECalBackendMAPI *cbmapi, icalcomponent *comp, struct cbdata *cb
 	cast_mapi_SPropValue (&(res.res.resProperty.lpProp), &sprop);
 
 	exchange_mapi_connection_fetch_items (priv->fid, &res, NULL,
-					NULL, 0, 
+					req_props_list, G_N_ELEMENTS (req_props_list),
 					NULL, NULL, 
 					capture_req_props, cbdata, 
 					MAPI_OPTIONS_FETCH_GENERIC_STREAMS);
@@ -1304,7 +1482,7 @@ e_cal_backend_mapi_create_object (ECalBackendSync *backend, EDataCal *cal, char
 	GSList *recipients = NULL;
 	GSList *attachments = NULL;
 	GSList *streams = NULL;
-	struct cbdata cbdata;
+	struct cbdata cbdata = { 0 };
 	struct Binary_r globalid;
 	struct icaltimetype current;
 
@@ -1500,7 +1678,7 @@ e_cal_backend_mapi_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	GSList *recipients = NULL;
 	GSList *streams = NULL;
 	GSList *attachments = NULL;
-	struct cbdata cbdata;
+	struct cbdata cbdata = { 0 };
 	gboolean no_increment = FALSE;
 	icalproperty *prop;
 	struct icaltimetype current;
@@ -1518,7 +1696,7 @@ e_cal_backend_mapi_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 
 	if (mod != CALOBJ_MOD_ALL) {
 		e_cal_backend_notify_error (E_CAL_BACKEND (cbmapi), _("Support for modifying single instances of a recurring appointment is not yet implemented. No change was made to the appointment on the server.")); 
-		return GNOME_Evolution_Calendar_Success;
+		return GNOME_Evolution_Calendar_OtherError;
 	}
 
 	/* check the component for validity */
@@ -1780,11 +1958,13 @@ e_cal_backend_mapi_send_objects (ECalBackendSync *backend, EDataCal *cal, const
 		icalcomponent *subcomp = icalcomponent_get_first_component (icalcomp, kind);
 		while (subcomp) {
 			ECalComponent *comp = e_cal_component_new ();
-			struct cbdata cbdata;
+			struct cbdata cbdata = { 0 };
 			mapi_id_t mid = 0;
 			GSList *recipients = NULL;
 			GSList *attachments = NULL;
 			GSList *streams = NULL;
+			const gchar *compuid;
+			struct Binary_r globalid;
 
 			e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (subcomp));
 
@@ -1850,6 +2030,11 @@ e_cal_backend_mapi_send_objects (ECalBackendSync *backend, EDataCal *cal, const
 			cbdata.get_timezone = (icaltimezone * (*)(gpointer data, const gchar *tzid)) e_cal_backend_mapi_internal_get_timezone;
 			cbdata.get_tz_data = cbmapi;
 
+			e_cal_component_get_uid (comp, &compuid);
+			exchange_mapi_cal_util_generate_globalobjectid (TRUE, compuid, &globalid);
+			cbdata.globalid = &globalid;
+			cbdata.cleanglobalid = &globalid;
+
 			mid = exchange_mapi_create_item (olFolderSentMail, 0,
 							exchange_mapi_cal_util_build_name_id, GINT_TO_POINTER(kind), 
 							exchange_mapi_cal_util_build_props, &cbdata, 
@@ -1927,8 +2112,19 @@ e_cal_backend_mapi_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 
 			switch (method) {
 			case ICAL_METHOD_REQUEST :
-				comp_str = e_cal_component_get_as_string (comp);
-				status = e_cal_backend_mapi_modify_object (backend, cal, comp_str, CALOBJ_MOD_THIS, &old_object, &new_object);
+				comp_str = NULL;
+				status = e_cal_backend_mapi_get_object (backend, cal, uid, NULL, &comp_str);
+				if (status != GNOME_Evolution_Calendar_Success) {
+					comp_str = e_cal_component_get_as_string (comp);
+					new_object = comp_str;
+					status = e_cal_backend_mapi_create_object (backend, cal, &new_object, NULL);
+					if (new_object == comp_str)
+						new_object = NULL;
+				} else {
+					g_free (comp_str);
+					comp_str = e_cal_component_get_as_string (comp);
+					status = e_cal_backend_mapi_modify_object (backend, cal, comp_str, CALOBJ_MOD_ALL, &old_object, &new_object);
+				}
 				g_free (comp_str);
 				g_free (old_object);
 				g_free (new_object);
@@ -1977,42 +2173,6 @@ e_cal_backend_mapi_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 	return status;
 }
 
-
-static ECalBackendSyncStatus 
-e_cal_backend_mapi_get_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzid, char **object)
-{
-	ECalBackendMAPI *cbmapi;
-	ECalBackendMAPIPrivate *priv;
-
-	icaltimezone *zone;
-	icalcomponent *icalcomp;
-
-	cbmapi = E_CAL_BACKEND_MAPI (backend);
-	priv = cbmapi->priv;
-
-	g_return_val_if_fail (tzid != NULL, GNOME_Evolution_Calendar_ObjectNotFound);
-
-	if (!strcmp (tzid, "UTC")) {
-		zone = icaltimezone_get_utc_timezone ();
-	} else {
-		zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
-
-		if (!zone && g_str_has_prefix (tzid, OLD_TZID_PREFIX))
-			zone = icaltimezone_get_builtin_timezone (tzid + strlen (OLD_TZID_PREFIX));
-
-		if (!zone)
-			return GNOME_Evolution_Calendar_ObjectNotFound;
-	}
-
-	icalcomp = icaltimezone_get_component (zone);
-	if (!icalcomp)
-		return GNOME_Evolution_Calendar_InvalidObject;
-
-	*object = icalcomponent_as_ical_string_r (icalcomp);
-
-	return GNOME_Evolution_Calendar_Success;
-}
-
 static ECalBackendSyncStatus 
 e_cal_backend_mapi_add_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj)
 {
@@ -2316,22 +2476,20 @@ e_cal_backend_mapi_internal_get_default_timezone (ECalBackend *backend)
 static icaltimezone *
 e_cal_backend_mapi_internal_get_timezone (ECalBackend *backend, const char *tzid)
 {
+	ECalBackendMAPI *cbmapi;
 	icaltimezone *zone;
 
+	cbmapi = E_CAL_BACKEND_MAPI (backend);
+
+	g_return_val_if_fail (cbmapi != NULL, NULL);
+	g_return_val_if_fail (cbmapi->priv != NULL, NULL);
 	g_return_val_if_fail (tzid != NULL, NULL);
-	g_return_val_if_fail (backend != NULL, NULL);
 
-	zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
-
-	if (!zone && g_str_has_prefix (tzid, OLD_TZID_PREFIX))
-		zone = icaltimezone_get_builtin_timezone (tzid + strlen (OLD_TZID_PREFIX));
+	zone = (icaltimezone *) e_cal_backend_cache_get_timezone (cbmapi->priv->cache, tzid);
 
 	if (!zone && E_CAL_BACKEND_CLASS (parent_class)->internal_get_timezone)
 		zone = E_CAL_BACKEND_CLASS (parent_class)->internal_get_timezone (backend, tzid);
 
-	if (!zone)
-		return icaltimezone_get_utc_timezone ();
-
 	return zone;
 }
 
@@ -2370,7 +2528,6 @@ e_cal_backend_mapi_class_init (ECalBackendMAPIClass *class)
 	sync_class->discard_alarm_sync = e_cal_backend_mapi_discard_alarm;
 	sync_class->receive_objects_sync = e_cal_backend_mapi_receive_objects;
 	sync_class->send_objects_sync = e_cal_backend_mapi_send_objects;
-	sync_class->get_timezone_sync = e_cal_backend_mapi_get_timezone;
 	sync_class->add_timezone_sync = e_cal_backend_mapi_add_timezone;
 	sync_class->set_default_zone_sync = e_cal_backend_mapi_set_default_zone;
 	sync_class->get_freebusy_sync = e_cal_backend_mapi_get_free_busy;
@@ -2386,7 +2543,7 @@ e_cal_backend_mapi_class_init (ECalBackendMAPIClass *class)
 
 
 static void
-e_cal_backend_mapi_init (ECalBackendMAPI *cbmapi, ECalBackendMAPIClass *class)
+e_cal_backend_mapi_init (ECalBackendMAPI *cbmapi)
 {
 	ECalBackendMAPIPrivate *priv;
 	
@@ -2403,30 +2560,6 @@ e_cal_backend_mapi_init (ECalBackendMAPI *cbmapi, ECalBackendMAPIClass *class)
 	e_cal_backend_sync_set_lock (E_CAL_BACKEND_SYNC (cbmapi), TRUE);
 }
 
-GType
-e_cal_backend_mapi_get_type (void)
-{
-	static GType e_cal_backend_mapi_type = 0;
-
-	if (!e_cal_backend_mapi_type) {
-		static GTypeInfo info = {
-                        sizeof (ECalBackendMAPIClass),
-                        (GBaseInitFunc) NULL,
-                        (GBaseFinalizeFunc) NULL,
-                        (GClassInitFunc) e_cal_backend_mapi_class_init,
-                        NULL, NULL,
-                        sizeof (ECalBackendMAPI),
-                        0,
-                        (GInstanceInitFunc) e_cal_backend_mapi_init
-                };
-		e_cal_backend_mapi_type = g_type_register_static (E_TYPE_CAL_BACKEND_SYNC,
-								  "ECalBackendMAPI", &info, 0);
-	}
-
-	return e_cal_backend_mapi_type;
-}
-
-
 /***** UTILITY FUNCTIONS *****/
 const char *	
 e_cal_backend_mapi_get_local_attachments_store (ECalBackendMAPI *cbmapi)
