@@ -561,14 +561,12 @@ mapi_update_cache (CamelFolder *folder, GSList *list, CamelFolderChangeInfo **ch
  				/* Build a SRow structure */
  				aRow = &recip->out_SRow;
 
- 				type = (uint32_t *) find_SPropValue_data (aRow, PR_RECIPIENT_TYPE);
+ 				type = (uint32_t *) exchange_mapi_util_find_row_propval (aRow, PR_RECIPIENT_TYPE);
 
 				if (type) {
- 					name = (const char *) find_SPropValue_data (aRow, PR_DISPLAY_NAME);
- 					name = name ? name : (const char *) find_SPropValue_data (aRow, PR_RECIPIENT_DISPLAY_NAME);
- 					name = name ? name : (const char *) find_SPropValue_data (aRow, 
- 												 PR_RECIPIENT_DISPLAY_NAME_UNICODE);
- 					name = name ? name : (const char *) find_SPropValue_data (aRow, 
+					name = (const gchar *) exchange_mapi_util_find_row_propval (aRow, PR_DISPLAY_NAME_UNICODE);
+					name = name ? name : (const gchar *) exchange_mapi_util_find_row_propval (aRow, PR_RECIPIENT_DISPLAY_NAME_UNICODE);
+					name = name ? name : (const gchar *) exchange_mapi_util_find_row_propval (aRow,
  												 PR_7BIT_DISPLAY_NAME_UNICODE);
  					display_name = name ? name : recip->email_id;
  					formatted_id = camel_internet_address_format_address(display_name, recip->email_id);
@@ -775,13 +773,18 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 		camel_operation_progress (NULL, (index * 100)/count); /* ;-) */
 
 		/* Check if we have to stop */
-		if (camel_operation_cancel_check(NULL))
+		if (camel_operation_cancel_check(NULL)) {
+			if (camel_folder_change_info_changed (changes))
+				camel_object_trigger_event (m->folder, "folder_changed", changes);
+			camel_folder_change_info_free (changes);
 			return;
+		}
 	}
 
 	camel_operation_end (NULL);
 
-	/* camel_object_trigger_event (m->folder, "folder_changed", changes); */
+	if (camel_folder_change_info_changed (changes))
+		camel_object_trigger_event (m->folder, "folder_changed", changes);
 	camel_folder_change_info_free (changes);
 
 	m->need_refresh = camel_folder_summary_count (m->folder->summary) != g_slist_length (server_uid_list);
@@ -988,28 +991,20 @@ camel_mapi_folder_fetch_summary (CamelStore *store, const mapi_id_t fid, struct
 
 	const guint32 summary_prop_list[] = {
 		PR_INTERNET_CPID,
-		PR_SUBJECT,
 		PR_SUBJECT_UNICODE,
 		PR_MESSAGE_SIZE,
 		PR_MESSAGE_DELIVERY_TIME,
 		PR_MESSAGE_FLAGS,
-		PR_SENT_REPRESENTING_NAME,
 		PR_SENT_REPRESENTING_NAME_UNICODE,
-		PR_SENT_REPRESENTING_EMAIL_ADDRESS,
 		PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE,
-		PR_SENT_REPRESENTING_ADDRTYPE,
 		PR_SENT_REPRESENTING_ADDRTYPE_UNICODE,
 		PR_LAST_MODIFICATION_TIME,
 		PR_INTERNET_MESSAGE_ID,
 		PR_INTERNET_REFERENCES,
 		PR_IN_REPLY_TO_ID,
-		PR_DISPLAY_TO,
 		PR_DISPLAY_TO_UNICODE,
-		PR_DISPLAY_CC,
 		PR_DISPLAY_CC_UNICODE,
-		PR_DISPLAY_BCC,
 		PR_DISPLAY_BCC_UNICODE,
-		PR_TRANSPORT_MESSAGE_HEADERS,
 		PR_TRANSPORT_MESSAGE_HEADERS_UNICODE
 	};
 
@@ -1172,19 +1167,14 @@ static const uint32_t camel_GetPropsList[] = {
 	PR_MID, 
 	PR_INTERNET_CPID,
 
-
-	PR_TRANSPORT_MESSAGE_HEADERS,
 	PR_TRANSPORT_MESSAGE_HEADERS_UNICODE,
 	PR_MESSAGE_CLASS, 
-	PR_MESSAGE_CLASS_UNICODE, 
 	PR_MESSAGE_SIZE, 
 	PR_MESSAGE_FLAGS, 
 	PR_MESSAGE_DELIVERY_TIME, 
 	PR_MSG_EDITOR_FORMAT, 
 
-	PR_SUBJECT, 
 	PR_SUBJECT_UNICODE, 
-	PR_CONVERSATION_TOPIC, 
 	PR_CONVERSATION_TOPIC_UNICODE, 
 
 	/*Properties used for message threading.*/
@@ -1199,11 +1189,8 @@ static const uint32_t camel_GetPropsList[] = {
  	/*PR_BODY_HTML, */
  	/*PR_BODY_HTML_UNICODE, */
 
-	PR_DISPLAY_TO, 
 	PR_DISPLAY_TO_UNICODE, 
-	PR_DISPLAY_CC, 
 	PR_DISPLAY_CC_UNICODE, 
-	PR_DISPLAY_BCC, 
 	PR_DISPLAY_BCC_UNICODE, 
 
 	PR_CREATION_TIME, 
@@ -1216,25 +1203,16 @@ static const uint32_t camel_GetPropsList[] = {
 	PR_OWNER_APPT_ID, 
 	PR_PROCESSED, 
 
-	PR_SENT_REPRESENTING_NAME, 
 	PR_SENT_REPRESENTING_NAME_UNICODE, 
-	PR_SENT_REPRESENTING_ADDRTYPE, 
 	PR_SENT_REPRESENTING_ADDRTYPE_UNICODE, 
-	PR_SENT_REPRESENTING_EMAIL_ADDRESS, 
 	PR_SENT_REPRESENTING_EMAIL_ADDRESS_UNICODE, 
 
-	PR_SENDER_NAME, 
 	PR_SENDER_NAME_UNICODE, 
-	PR_SENDER_ADDRTYPE, 
 	PR_SENDER_ADDRTYPE_UNICODE, 
-	PR_SENDER_EMAIL_ADDRESS, 
 	PR_SENDER_EMAIL_ADDRESS_UNICODE, 
 
-	PR_RCVD_REPRESENTING_NAME, 
 	PR_RCVD_REPRESENTING_NAME_UNICODE, 
-	PR_RCVD_REPRESENTING_ADDRTYPE, 
 	PR_RCVD_REPRESENTING_ADDRTYPE_UNICODE, 
-	PR_RCVD_REPRESENTING_EMAIL_ADDRESS, 
 	PR_RCVD_REPRESENTING_EMAIL_ADDRESS_UNICODE
 };
 
@@ -1371,7 +1349,7 @@ fetch_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 
 		if (appointment_body_str && *appointment_body_str) {
 			body = g_new0(ExchangeMAPIStream, 1);
-			body->proptag = PR_BODY;
+			body->proptag = PR_BODY_UNICODE;
 			body->value = g_byte_array_new ();
 			body->value = g_byte_array_append (body->value, appointment_body_str, strlen ((const char *)appointment_body_str));
 
@@ -1443,12 +1421,11 @@ mapi_mime_set_recipient_list (CamelMimeMessage *msg, MapiItem *item)
 		aRow = &recip->out_SRow;
 		
 		/*Name is probably available in one of these props.*/
-		name = (const char *) find_SPropValue_data (aRow, PR_DISPLAY_NAME);
-		name = name ? name : (const char *) find_SPropValue_data (aRow, PR_RECIPIENT_DISPLAY_NAME);
-		name = name ? name : (const char *) find_SPropValue_data (aRow, PR_RECIPIENT_DISPLAY_NAME_UNICODE);
-		name = name ? name : (const char *) find_SPropValue_data (aRow, PR_7BIT_DISPLAY_NAME_UNICODE);
+		name = (const gchar *) exchange_mapi_util_find_row_propval (aRow, PR_DISPLAY_NAME_UNICODE);
+		name = name ? name : (const gchar *) exchange_mapi_util_find_row_propval (aRow, PR_RECIPIENT_DISPLAY_NAME_UNICODE);
+		name = name ? name : (const gchar *) exchange_mapi_util_find_row_propval (aRow, PR_7BIT_DISPLAY_NAME_UNICODE);
 
-		type = (uint32_t *) find_SPropValue_data (aRow, PR_RECIPIENT_TYPE);
+		type = (uint32_t *) exchange_mapi_util_find_row_propval (aRow, PR_RECIPIENT_TYPE);
 		
 		/*Fallbacks. Not good*/
 		display_name = name ? g_strdup (name) : g_strdup (recip->email_id);
@@ -1590,7 +1567,15 @@ mapi_mime_msg_body (MapiItem *item, const ExchangeMAPIStream *body)
 				"text/plain" : "text/html";
 
 		if (item->header.cpid) {
-			if (item->header.cpid == 65001)
+			if (item->header.cpid >= 28591 && item->header.cpid <= 28599)
+				buff = g_strdup_printf ("%s; charset=\"ISO-8859-%d\"", type, item->header.cpid % 10);
+			else if (item->header.cpid == 28603)
+				buff = g_strdup_printf ("%s; charset=\"ISO-8859-13\"", type);
+			else if (item->header.cpid == 28605)
+				buff = g_strdup_printf ("%s; charset=\"ISO-8859-15\"", type);
+			else if (item->header.cpid == 65000)
+				buff = g_strdup_printf ("%s; charset=\"UTF-7\"", type);
+			else if (item->header.cpid == 65001)
 				buff = g_strdup_printf ("%s; charset=\"UTF-8\"", type);
 			else
 				buff = g_strdup_printf ("%s; charset=\"CP%d\"", type, item->header.cpid);
@@ -1740,11 +1725,11 @@ mapi_mime_classify_attachments (GSList *attachments, GSList **inline_attachs, GS
 		part = camel_mime_part_new ();
 
 		filename = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, 
-											 PR_ATTACH_LONG_FILENAME);
+											 PR_ATTACH_LONG_FILENAME_UNICODE);
 
 		if (!(filename && *filename))
 			filename = (const char *) exchange_mapi_util_find_SPropVal_array_propval(attach->lpProps, 
-												 PR_ATTACH_FILENAME);
+												 PR_ATTACH_FILENAME_UNICODE);
 		camel_mime_part_set_filename(part, g_strdup(filename));
 		camel_content_type_set_param (((CamelDataWrapper *) part)->mime_type, "name", filename);
 
